#!/bin/bash
# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2009, 2011 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplicate or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.

trap exit_cleanup 1 2 3 5 9 11 15

export TEXTDOMAINDIR=/usr/share/locale
export TEXTDOMAIN=backup
export PATH=/usr/bin:/sbin:/usr/sbin:/bin:$PATH
RUN_AS_ROOT=/opt/ibm/director/vmi/bin/execWithPrivilege
MGMTNETUTIL=mgmtNetUtil
SNAPSHOT="runcommand2 /opt/ibm/directorappliance/bin/backupvm.sh 'snapshot'"
CLEAN="runcommand2 /opt/ibm/directorappliance/bin/backupvm.sh 'clean'"
COMMAND_NAME=backup
ARGS="$*"
Y="%Y"
H="%H"
M="%M"
S="%S"
FILENAME="`date +${Y}%m%d${H}${M}${S}`"
TYPE=
SFTPSERVER=
SFTPDIR=
USER=""
PASSWORD=""
ANSWER="empty"
doUsbClean=1
doSmstop=0

# source log functions
# logUti has the following functions:
# start_logging, stop_logging, logmsg, logoutput, logprintf
logUtil=/opt/ibm/director/vmi/scripts/execWithPrivilege/logUtil
source $logUtil
logfilename=/var/log/messages_backup
logfilesize=1048576


# plugins
plugindir=/opt/ibm/vmi/customize/backup.d/

exit_cleanup()
{
   logoutput $RUN_AS_ROOT flagsbk clear lock
   logoutput $RUN_AS_ROOT flagsbk clear postrestore

   if [ "$1" == "0" ]; then
      /opt/ibm/director/vmi/bin/logevent -t success -c $COMMAND_NAME -a "$ARGS"
   else
      /opt/ibm/director/vmi/bin/logevent -t failed -c $COMMAND_NAME -a "$ARGS"
      	# call cleanup plugins scripts if fails
	logmsg "call cleanup plugins scripts if fails"
	logerror $run_as_root runPlugins $plugindir backup cleanup 
   fi
   logoutput $RUN_AS_ROOT $MGMTNETUTIL $CLEAN
   if [ "$doUsbClean" == "0" ]; then
	# umount drive in host, sync in host, drop all file system caches in guest
        logoutput $RUN_AS_ROOT usbUtil clean
   fi
   stop_logging
   exit $1
}

print_usage() {
	logmsg "INFO enter print_usage"
        printf "`gettext 'Usage: backup  -l {usb | sftp}\n\t\t-s {sftp_server}\n\t\t-d {path}\n\t\t-u {user}\n\t\t[-p {password}]\n\t\t[-Y]\n\t\t[-h | --help]'`\n\n
"
        printf "`gettext 'Back up the system.'`\n\n"
        printf "`gettext '  -l\t\t- The location to store the backup file.\n\t\tIf sftp is specified, argument -s is required.'`\n"
        printf "`gettext '  -s\t\t- Specify SFTP server. Required when -l sftp is sepcified.'`\n"
        printf "`gettext '  -d\t\t- The directory path in the SFTP server to save the backup file. Example: -d /data/backup'`\n"
        printf "`gettext '  -u\t\t- The SFTP user account.'`\n"
        printf "`gettext '  -p\t\t- The SFTP user account password.'`\n"
        printf "`gettext '  -Y\t\t- Answer yes to all promoted questions.'`\n"
        printf "`gettext '  -h|--help\t- Print this help'`\n\n"
        printf "`gettext 'Example:'`\n\n"
        printf "`gettext '  backup -l usb -Y'`\n"
        printf "`gettext '\t\t-backup the whole system to USB device without prompt.'`\n\n"
        printf "`gettext '  backup -l sftp -s sftp_server -d path_on_sftp -u user -p password'`\n"
        printf "`gettext '\t\t-backup the whole system to SFTP server.'`\n\n"

  	 
  exit_cleanup 0
}

# run HA team provided script to check if the HA is runing
# return value: 1 = HA is runing, 0 = HA isn't runing
is_HA_running()
{
    logmsg "INFO enter is_HA_running"
    logmsg "INFO smhapstatus result"
    logoutput $RUN_AS_ROOT smhapstatus.sh
    case $? in
        0|3) return 1;;
        2|4) return 0;;
    esac
    return 0
}

# check smstatus to see if director is running
# smstatus=Active(0), director is running, need to call smstop
# smstatus=Starting(1), director is starting, need to stop backup
# smstatus=Ending(2), director is ending, need to wait for a while
#                     if director is still ending, stop backup
#                     if director is inactive, continue backup
# smstatus=Inactive(3), director is Inactive, continue backup
#
# return value: 
#     0=Inactive(continue backup)
#     1=Active(call smstop to before snapshot, call smstart after snapshot)
#     2=Starting/Stopping(stop backup)
is_director_running(){
    logmsg "INFO enter is_director_running"
    logoutput smstatus
    rc=$?
    case $rc in
	    0) return 1;;  # Active
    	    3) return 0;;  # Inactive
            1) return 2;;  # Starting
            2)  # Ending
		wait_num=0
		while [ $wait_num -lt 3 ];do
			wait_num=$((wait_num+1))
			sleep 2
			logmsg "INFO wait_number is $wait_num"
			logoutput smstatus
			logprintf "`gettext 'Waiting for Director to stop.'`\n"
			if [ $? -eq 3 ]; then # from Ending to Inactive
				return 0
			fi
		done
		# Still Ending
		return 2
		;;
    esac
}

# run the plugins and exits if a bad rc is encountered.
runPlugin() {
	logmsg "INFO Enter runPlugin"
	exitCode=$1
	shift
	logerror $RUN_AS_ROOT runPlugins $plugindir $*
	rc=$?
	logmsg "INFO the return code of plugin is $rc"
  	if [ $rc -ne 0 ]; then
    		exit_cleanup $exitCode
  	fi
}

$RUN_AS_ROOT logUtil rotate $logfilename $logfilesize
start_logging

# Check if it's HW appliance, this command can only be run in HW appliance
logmsg "INFO Check if it's HW appliance"
getSpecProperty=/opt/ibm/director/vmi/scripts/execWithPrivilege/getSpecProperty
hwapp=$($getSpecProperty network.hwapp.enabled)

logmsg "INFO Mark lock"
logoutput $RUN_AS_ROOT flagsbk mark lock
if [ "$?" -eq "1" ]; then
	logmsg "ERROR Backup or restore command is already running."
	logprintf "`gettext 'Backup or restore command is already running.'`\n"
  	exit 16 # don't cleanup. director may not be running
fi

if [ "$?" -eq "0" -a "$hwapp" == "yes" ]; then
        # Test HA, this command can only run in non-HA applaince
        is_HA_running
        if [ $? -eq 1 ]; then
	  logmsg "ERROR This command is not supported when HA is configured."
          logprintf "`gettext 'This command is not supported when HA is configured.'`\n"
          exit_cleanup 15
        fi

	logmsg "INFO HA is not configured"
	
	# Test authorization, exit 3 if authorization error occurrs.
	logmsg "INFO run checkpermission"

	logoutput /opt/ibm/director/vmi/scripts/execWithPrivilege/checkpermission $COMMAND_NAME
	if [ $? -ne 0 ]; then
	  logmsg "ERROR The user ID does not have the authority to run the backup command."
	  logprintf "`gettext 'The user ID does not have the authority to run the backup command.'`\n"
	  exit_cleanup 3
	fi
	
	logmsg "INFO parameters $*"

	if [ "$#" -lt "1" ]; then
	   print_usage
	fi

	ARGS=`getopt -ua -n $COMMAND_NAME -l help -o "l:s:d:u:p:h:Y" -- $*`
	[[ $? -ne 0 ]] && exit_cleanup 1
	set -- $ARGS
	
	# Parse the arguments
	while [ "$1" != "--" ]
	do
		case "$1" in
			-h|--help) print_usage;;
			-l) 
				shift
				case "$1" in
					usb) 
						TYPE=$1
						;;					
					sftp) 
						TYPE=$1
						;;
					*) print_usage;;
				esac;;
			-s)
				shift
				SFTPSERVER=$1
				;;
			-d)
				shift
				SD=$1
				SFTPDIR=${SD/%\//}
				;;
			-u)
				shift
				USER=$1
				;;
			-p)
				shift
				PASSWORD=$1
				;;
			-Y)
				ANSWER="y"
				;;
			*) print_usage;;
		esac
		shift
	done

	if [ "$TYPE" != "sftp" -a "$TYPE" != "usb" ]; then
		print_usage
	fi
	
	if [ "$TYPE" = "sftp" -a "$SFTPSERVER" = "" ]; then
		print_usage
	fi

	if [ "$TYPE" = "sftp" -a "$USER" = "" ]; then
		print_usage
	fi

	if [ "$TYPE" = "sftp" -a "$SFTPDIR" = "" ]; then
  		print_usage
	fi

	if [ "$TYPE" = "sftp" -a "$USER" != "" -a "$PASSWORD" = "" ]; then
		echo "Input password for SFTP"
		stty -echo #screen the input so that no one will see what u type
		read PASSWORD
		stty echo
	fi
	
	if [ "$TYPE" = "usb" -a "$SFTPSERVER" != "" ]; then
		print_usage
	fi

	# run plugins to determine if backup can be executed at current env
	logmsg "INFO Run plugins to determine if backup can be executed at current env"
	runPlugin 17 backup validate 


	# Test the USB or SFTP is ready before start the real backup
	if [ "$TYPE" = "usb" ]; then
		logmsg "INFO USB backup"
		logmsg "INFO run lsmediadev"
		if ! lsmediadev | grep -i type=3 2>&1 >/dev/null; then
			logmsg "ERROR A usable USB disk was not found. Check the installation and try again."
			logprintf "`gettext 'A usable USB disk was not found. Check the installation and try again.'`\n"
			exit_cleanup 8
		fi
	elif [ "$TYPE" = "sftp" ]; then
		logmsg "INFO SFTP backup"
		logmsg "INFO run sftptestbk.exp"
		logoutput $RUN_AS_ROOT sftptestbk.exp $SFTPSERVER $SFTPDIR/$FILENAME.tar.gz $USER $PASSWORD
		rc=$?
		logmsg "INFO the return of sftptestbk.exp is $rc"
	  	if [ $rc -ne 0 ]; then
	  		if [ $rc -eq 9 ]; then
				logmsg "ERROR Unable to communicate with the SFTP server."
	  			logprintf "`gettext 'Unable to communicate with the SFTP server.'`\n"
	  			exit_cleanup 9
	  		elif [ $rc -eq 10 ]; then
				logmsg "ERROR Authentication with the SFTP server failed. Check the user name and password."
	  		  	logprintf "`gettext 'Authentication with the SFTP server failed. Check the user name and password.'`\n"
	  	  		exit_cleanup 10
	  		elif [ $rc -eq 11 ]; then
				logmsg "ERROR Please check the file path option, and try again."
	  	  		logprintf "`gettext 'Please check the file path option, and try again.'`\n"
	  	  		exit_cleanup 11
	  		elif [ $rc -eq 12 ]; then
				logmsg "ERROR Host key verification failed."
        			logprintf "`gettext 'Host key verification failed.'`\n"
        			exit_cleanup 12
      			else
				logmsg "ERROR Unexpected error."
        			logprintf "`gettext 'Unexpected error.'`\n"
        			exit_cleanup 13
	    		fi
	  	fi
	fi
	
	while [ $ANSWER != "y" ] && [ $ANSWER != "Y" ] && [ $ANSWER != "yes" ]; do
		logmsg "INFO The Systems Director server will be stopped while performing the backup, continue?  ([y]es or [n]o?)"		
	  	logprintf "`gettext 'The Systems Director server will be stopped while performing the backup, continue?  ([y]es or [n]o?)'`"
	  	read ANSWER
		logmsg "INFO ANSWER is $ANSWER"
	  	if [ -z $ANSWER ]; then
	  		ANSWER="empty"
	  	fi
	  
      		if [ $ANSWER == "n" ] || [ $ANSWER == "N" ] || [ $ANSWER == "no" ]; then
			logmsg "INFO exit because answer is no"
        		exit_cleanup 99
      		fi
	done

	# stop director if smstatus is Active
	# stop backup if smstatus is Starting
	# if smstatus is Ending, wait for director to because inactive for a while, continue backup if stopped, stop backup if smstatus is still Ending


	logmsg "INFO Check smstatus before taking snapshot"
	is_director_running
	rc=$?
        if [ $rc -eq 1 ]; then  # director is Active
		logmsg "INFO Director is Active"
	  	logerror smstop
		logmsg "INFO Called smstop. Loop to check if Director is stopped."
		wait_num=0
		stopped=0
		while [ $wait_num -lt 3 ];do
			wait_num=$((wait_num+1))
			logmsg "INFO wait_number is $wait_num"
			logoutput smstatus
			if [ $? -eq 3 ]; then # Director is stopped
				stopped=1
				break
			fi
			sleep 2
		done
		if [ $stopped -eq 0 ]; then # Director is not stopped by smstop
			logmsg "INFO Director can not be stopped"
			logprintf "`gettext 'Director can not be stopped, please use smstatus to check the status of Director and backup after Director is stopped.'`\n"
		exit_cleanup 19
		fi
		doSmstop=1
  	elif [ $rc -eq 2 ]; then # director is starting or ending
		logmsg "INFO Director is Starting or Ending"
		logprintf "`gettext 'Director is Starting or Ending, please start backup after Director is Active or Inactive,'`\n"
		exit_cleanup 18
        fi
	
        # Mark post restore flag
        logmsg "INFO Mark post restore flag"
        logoutput $RUN_AS_ROOT flagsbk mark postrestore

	# Make a manifest file to host
	logmsg "INFO write manifest"
	logoutput $RUN_AS_ROOT manifest

	logmsg "INFO sync disk"
        logoutput $RUN_AS_ROOT syncdisk

	# call plugins scripts before snapshot
	logmsg "INFO call plugins scripts before snapshot"
	runPlugin 17 backup pre
	
	logmsg "INFO make snapshot"
	logoutput $RUN_AS_ROOT $MGMTNETUTIL $SNAPSHOT

	if [ $? -ne 0 ]; then
		logmsg "ERROR An internal error occurred. Failed to create the snapshot"
		logprintf "`gettext 'An internal error occurred. Failed to create the snapshot'`\n"
		exit_cleanup 2
	fi

	# clear post restore flag
	logmsg "INFO clear post restore flag"
	logoutput $RUN_AS_ROOT flagsbk clear postrestore

	# call plugins scripts after snapshot
	logmsg "info call plugins scripts after snapshot"
	runPlugin 17 backup post

	if [ $doSmstop -eq 1 ]; then
		logmsg "info call smstart, start director"
		logerror smstart 
	fi

	if [ "$TYPE" = "usb" ]; then
                # prepare appliance for usb backup, umount drive in guest, sync in guest, drop all file system caches in host
		logmsg "INFO prepare appliance for usb backup"
                logoutput $RUN_AS_ROOT usbUtil prepare
		rc=$?

		# since we are doing a usb backup, we should clean the usb after backup, even after failure
		doUsbClean=0
		if [ $rc -ne 0 ]; then
			logmsg "ERROR The USB device is busy."
			logprintf "`gettext 'The USB device is busy.'`\n"
			exit_cleanup 16
		fi

		trap "" 1
                # execute usb backup
                BACKUPVM="runcommand2 nice -n 19 ionice -c 3 /opt/ibm/directorappliance/bin/backupvm.sh 'usb' $FILENAME"
		logmsg "INFO start backup, the backup file is $FILENAME.tar.gz"
		logprintf "`gettext \"Performing the back up. The filename is $FILENAME.tar.gz, please wait...\"`\n"
                logoutput $RUN_AS_ROOT $MGMTNETUTIL $BACKUPVM    
		rc=$?
		logmsg "INFO the return of calling backupvm.sh usb is $rc"
		if [ $rc -ne 0 ]; then
			if [ $rc -eq 4 ]; then
				logmsg "INFO check usb space"
				logoutput $RUN_AS_ROOT usbUtil checkspace
				if [ $? -ne 0 ]; then
					logmsg "ERROR Failed to write to USB disk. There is not enough space left on USB disk."
					logprintf "`gettext 'Failed to write to USB disk. There is not enough space left on USB disk.'`\n"
					logoutput $RUN_AS_ROOT $MGMTNETUTIL runcommand2 rm -f /mnt/usb/$FILENAME.tar.gz.[a-z][a-z]
				else
					logmsg "ERROR Failed to write to USB disk, please check it."
					logprintf "`gettext 'Failed to write to USB disk, please check it.'`\n"			
				fi
				exit_cleanup 4
			elif [ $rc -eq 5 ]; then
				logmsg "ERROR A usable USB disk was not found. Check the installation and try again."
				logprintf "`gettext 'A usable USB disk was not found. Check the installation and try again.'`\n"
				exit_cleanup 5
			elif [ $rc -eq 14 ]; then
				logmsg "ERROR Unable to mount USB disk.."
				logprintf "`gettext 'Unable to mount USB disk..'`\n"
				exit_cleanup 14
      			else
				logmsg "ERROR Unexpected error."
        			logprintf "`gettext 'Unexpected error.'`\n"
        			exit_cleanup 13
			fi
		fi
		trap exit_cleanup 1
		# umount drive in host, sync in host, drop all file system caches in guest
		logmsg "INFO clean appliance after usb backup"
                logoutput $RUN_AS_ROOT usbUtil clean

	elif [ "$TYPE" = "sftp" ]; then	
		logmsg "INFO start backup, the backup file is $FILENAME.tar.gz"
		logprintf "`gettext \"Performing the back up. The filename is $FILENAME.tar.gz, please wait...\"`\n"
		logoutput $RUN_AS_ROOT transferbk.sh $SFTPSERVER $SFTPDIR/$FILENAME.tar.gz $USER $PASSWORD
		if [ $? -ne 0 ]; then
			logmsg "ERROR Failed to transfer the backup file to the SFTP server."
			logprintf "`gettext 'Failed to transfer the backup file to the SFTP server.'`\n" 
			exit_cleanup 6
		fi
	fi
	
	logmsg "INFO The backup completed successfully."
	logprintf "`gettext 'The backup completed successfully.'`\n"
	
else
	logmsg "INFO Command not available in this configuration. Contact the product supplier for details."
	logprintf "`gettext 'Command not available in this configuration. Contact the product supplier for details.'`\n"
fi

exit_cleanup 0
