#!/bin/bash
# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2009, 2011 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplicate or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.
trap exit_cleanup  1 2 3 5 9 11 15

export TEXTDOMAINDIR=/usr/share/locale
export TEXTDOMAIN=restore
export PATH=/usr/bin:/sbin:/usr/sbin:/bin:$PATH
RUN_AS_ROOT=/opt/ibm/director/vmi/bin/execWithPrivilege
MGMTNETUTIL=mgmtNetUtil
HOST_WATCHDOG=/opt/ibm/directorappliance/bin/watchdog-mgr
CLEAN="runcommand2 /opt/ibm/directorappliance/bin/restorevm.sh 'clean'"
COMMAND_NAME=restore
ARGS="$*"
TYPE=
SFTPSERVER=
FILENAME=
USER=""
PASSWORD=""
DIRNAME=
ANSWER="empty"

# source log functions
# logUti has the following functions:
# start_logging, stop_logging, logmsg, logoutput, logprintf
logUtil=/opt/ibm/director/vmi/scripts/execWithPrivilege/logUtil
source $logUtil
logfilename=/var/log/messages_restore
logfilesize=1048576

# plugins
plugindir=/opt/ibm/vmi/customize/backup.d/


exit_cleanup() {
   logoutput $RUN_AS_ROOT flagsbk clear lock
   if [ "$1" == "0" ]; then
      	/opt/ibm/director/vmi/bin/logevent -t success -c $COMMAND_NAME -a "$ARGS"
   else
     	/opt/ibm/director/vmi/bin/logevent -t failed -c $COMMAND_NAME -a "$ARGS"
     	logoutput $RUN_AS_ROOT $MGMTNETUTIL $CLEAN
      	# call cleanup plugins scripts if fails
	logmsg "call cleanup plugins scripts if fails"
	logerror $run_as_root runPlugins $plugindir restore cleanup 
   fi
   stop_logging
   exit $1
}

print_usage() {
	logmsg "INFO enter print_usage"
	printf "`gettext 'Usage: restore  -l {usb | sftp}\n\t\t-s {sftp_server}\n\t\t-n {file_name}\n\t\t-u {user}\n\t\t[-p {password}]\n\t\t[-Y]\n\t\t[-h | --help]'`\n\n"
	printf "`gettext 'Restore the system.'`\n\n"
	printf "`gettext '  -l\t\t- The location to store the backup file.\n\t\tIf sftp is specified, argument -s is needed to set the SFTP server.'`\n"
	printf "`gettext '  -s\t\t- Specify SFTP server when -l sftp is sepcified.'`\n"
	printf "`gettext '  -n\t\t- For SFTP server, specify the restore file name with the abstract path on the server, e.g. /path_on_ftp/20110110131235.tar.gz'`\n"
	printf "`gettext '    \t\t- For USB driver, specify the common part [YYYYmmddHHMMSS].tar.gz of the restore file names'`\n"
	printf "`gettext '    \t\t                 e.g. 20110110131235.tar.gz of file names 20110110131235.tar.gz.aa, 20110110131235.tar.gz.ab, etc...'`\n"
	printf "`gettext '  -u\t\t- The user to connect the SFTP server.'`\n"
	printf "`gettext '  -p\t\t- The password to connect the SFTP server.'`\n"
        printf "`gettext '  -Y\t\t- Answer yes to all promoted questions.'`\n"
	printf "`gettext '  -h|--help\t- Print this help'`\n\n"
        printf "`gettext 'Example:'`\n\n"
        printf "`gettext '  restore -l usb -n 20110110131235.tar.gz'`\n"
        printf "`gettext '\t\t-restore the whole system from the backup files 20110110131235.tar.gz.aa, 20110110131235.tar.gz.ab, etc... on USB device.'`\n\n"
        printf "`gettext '  restore -l sftp -s sftp_server -n /path_on_ftp/20110110131235.tar.gz -u user -p password'`\n"
        printf "`gettext '\t\t-restore the whole system from SFTP server.'`\n\n"
  exit_cleanup 0;
}

# run HA team provided script to check if the HA is runing
# return value: 1 = HA is runing, 0 = HA isn't runing
is_HA_running()
{
    logmsg "INFO enter is_HA_running"
    logmsg "INFO smhapstatus result"
    logoutput $RUN_AS_ROOT smhapstatus.sh
    case $? in
        0|3) return 1;;
        2|4) return 0;;
    esac
    return 0
}

# run the plugins and exits if a bad rc is encountered.
runPlugin() {
	logmsg "INFO Enter runPlugin"
	exitCode=$1
	shift
	logerror $RUN_AS_ROOT runPlugins $plugindir $*
	rc=$?
	logmsg "INFO the return code of plugin is $rc"
  	if [ $rc -ne 0 ]; then
    		exit_cleanup $exitCode
  	fi
}

$RUN_AS_ROOT logUtil rotate $logfilename $logfilesize
start_logging

# Check if it's HW appliance, this command can only be run in HW appliance
logmsg "INFO Check if it's HW appliance"
getSpecProperty=/opt/ibm/director/vmi/scripts/execWithPrivilege/getSpecProperty
hwapp=$($getSpecProperty network.hwapp.enabled)

logmsg "INFO Mark lock"
logoutput $RUN_AS_ROOT flagsbk mark lock
if [ "$?" -eq "1" ]; then
	logmsg "ERROR Backup or restore command is already running."
  	logprintf "`gettext 'Backup or restore command is already running.'`\n"
  	exit 16 # don't cleanup. director may not be running
fi

if [ "$?" -eq "0" -a "$hwapp" == "yes" ]; then

	# Test HA, this command can only run in non-HA applaince
        is_HA_running
        if [ $? -eq 1 ]; then
		logmsg "ERROR This command is not supported when HA is configured."
          	logprintf "`gettext 'This command is not supported when HA is configured.'`\n"
          	exit_cleanup 15
        fi

	logmsg "INFO HA is not configured"

	# Test authorization, exit 3 if authorization error occurrs.
	logmsg "INFO run checkpermission"
	logoutput /opt/ibm/director/vmi/scripts/execWithPrivilege/checkpermission $COMMAND_NAME
	if [ $? -ne 0 ]; then
		logmsg "ERROR The user ID does not have the authority to run the restore command."
	  	logprintf "`gettext 'The user ID does not have the authority to run the restore command.'`\n"
	  	exit_cleanup 3
	fi

	logmsg "INFO parameters $*"
	
	if [ "$#" -lt "1" ]; then
	   print_usage
	fi
	
	ARGS=`getopt -ua -n $COMMAND_NAME -l help -o "l:s:n:u:p:h:Y" -- $*`
	[[ $? -ne 0 ]] && exit_cleanup 1
	set -- $ARGS
	
	# Parse the arguments
	while [ "$1" != "--" ]
	do
		case "$1" in
			-h|--help) print_usage;;
			-l) 
				shift
				case "$1" in
					usb) 
							TYPE=$1
							;;					
					sftp) 
							TYPE=$1
							;;
					local)
							TYPE=$1
							;;
					*) print_usage;;
				esac;;
			-s)
				shift
				SFTPSERVER=$1
				;;
			-n) 
				shift
				FILENAME=$1
				;;
			-u)
				shift
				USER=$1
				;;
			-p)
				shift
				PASSWORD=$1
				;;
			-Y)
				ANSWER="y"
				;;
			*) print_usage;;
		esac
		shift
	done
	if [ "$TYPE" != "sftp" -a "$TYPE" != "usb" ]; then
		print_usage
	fi
	
	if [ "$TYPE" = "sftp" -a "$SFTPSERVER" = "" ]; then
		print_usage
	fi

	if [ "$TYPE" = "sftp" -a "$USER" = "" ]; then
		print_usage
	fi

	if [ "$FILENAME" = "" ]; then
		print_usage
	fi

	if [ "$TYPE" = "sftp" -a "$USER" != "" -a "$PASSWORD" = "" ]; then
		echo "Input password for SFTP"
		stty -echo #screen the input so that no one will see what u type
		read PASSWORD
		stty echo
	fi
	
	if [ "$TYPE" = "usb" -a "$SFTPSERVER" != "" ]; then
		print_usage
	fi

	if [ "$TYPE" = "local" -a "$SFTPSERVER" != "" ]; then
		print_usage
	fi

	# run plugins to determine if backup can be executed at current env
	logmsg "INFO Run plugins to determine if backup can be executed at current env"
	runPlugin 17 restore validate
	
	# Test the USB or SFTP is ready before start the real restore
	if [ "$TYPE" = "usb" ]; then
		logmsg "INFO USB restore"
		logmsg "INFO run lsmediadev"
		if ! lsmediadev | grep -i type=3 2>&1 >/dev/null; then
			logmsg "ERROR A usable USB disk was not found. Check the installation and try again."
			logprintf "`gettext 'A usable USB disk was not found. Check the installation and try again.'`\n"
			exit_cleanup 8
		fi
        elif [ "$TYPE" = "sftp" ]; then
		logmsg "INFO SFTP restore"
		logmsg "INFO run sftptestrt.exp"
                logoutput $RUN_AS_ROOT sftptestrt.exp $SFTPSERVER $FILENAME $USER $PASSWORD
                rc=$?
		logmsg "INFO the return of sftptestrt.exp is $rc"
                if [ $rc -ne 0 ]; then
                        if [ $rc -eq 9 ]; then
				logmsg "ERROR Unable to communicate with the SFTP server."
                                logprintf "`gettext 'Unable to communicate with the SFTP server.'`\n"
                                exit_cleanup 9
                        elif [ $rc -eq 10 ]; then
				logmsg "ERROR Authentication with the SFTP server failed. Check the user name and password."
                                logprintf "`gettext 'Authentication with the SFTP server failed. Check the user name and password.'`\n"
                                exit_cleanup 10
                        elif [ $rc -eq 11 ]; then
				logmsg "ERROR Please check the file path option, and try again."
                                logprintf "`gettext 'Please check the file path option, and try again.'`\n"
                                exit_cleanup 11
                        else
				logmsg "ERROR Unexpected error."
                                logprintf "`gettext 'Unexpected error.'`\n"
                                exit_cleanup 13
                        fi
                fi
        fi

	
	
	while [ $ANSWER != "y" ] && [ $ANSWER != "Y" ] && [ $ANSWER != "yes" ]; do
		logmsg "INFO The Systems Director server will be restarted while restore, continue?  ([y]es or [n]o?)"
	  	logprintf "`gettext 'The Systems Director server will be restarted while restore, continue?  ([y]es or [n]o?)'`"
		read ANSWER
		logmsg "INFO ANSWER is $ANSWER"
	  	if [ -z $ANSWER ]; then
	  		ANSWER="empty"
	  	fi
      		if [ $ANSWER == "n" ] || [ $ANSWER == "N" ] || [ $ANSWER == "no" ]; then
			logmsg "INFO exit because answer is no"
        		exit_cleanup 99
	  	fi
	done

	# call plugins scripts before snapshot
	logmsg "INFO call plugins scripts before restore"
	runPlugin 17 restore pre

	# create a path to store backup file
	logmsg "INFO create restore path"
       	CREATEPATH="runcommand2 /opt/ibm/directorappliance/bin/restorevm.sh 'createRestorePath'"	
	logoutput $RUN_AS_ROOT $MGMTNETUTIL $CREATEPATH
	rc=$?
	logmsg "INFO the return code of restorevm.sh createRestorePath is $rc"
	if [ $rc -ne 0 ]; then
		logmsg "ERROR Fail to create the restore path."
		logprintf "`gettext 'Fail to create the restore path.'`\n"
		exit_cleanup 30
	fi

	usbCleanFlag=0
	
	# Start restore
	if [ "$TYPE" = "usb" ]; then
		# prepare appliance for usb restore, umount drive in guest, sync in guest, drop all file system caches in host
		logmsg "INFO prepare appliance for usb restore"
                logoutput $RUN_AS_ROOT usbUtil prepare
		if [ $? -ne 0 ]; then
			logmsg "ERROR The USB device is busy."
			logprintf "`gettext 'The USB device is busy.'`\n"
			exit_cleanup 16
		fi

		RESTORE="runcommand2 nice -n 10 /opt/ibm/directorappliance/bin/restorevm.sh 'usb' $FILENAME"
		logmsg "INFO perform the restore"
		logprintf "`gettext 'Performing the restore, please wait...'`\n"
		logoutput $RUN_AS_ROOT $MGMTNETUTIL $RESTORE
		rc=$?
		logmsg "INFO the return code of restorevm.sh usb is $rc"
		if [ $rc -ne 0 ]; then
			if [ $rc -eq 8 ]; then
				logmsg "ERROR Unable to mount USB disk.."
				logprintf "`gettext 'Unable mount USB driver..'`\n"
				exit_cleanup 14
			elif [ $rc -eq 9 ]; then
				logmsg "ERROR Unable to copy file from USB driver"
				logprintf "`gettext 'Unable to copy file from USB driver, please check it and try again.'`\n"
				exit_cleanup 20 
			else
				logmsg "ERROR A usable USB disk was not found. Check the installation and try again."
				logprintf "`gettext 'A usable USB disk was not found. Check the installation and try again.'`\n"
				exit_cleanup 8
			fi
		fi

		usbCleanFlag=1

	elif [ "$TYPE" = "sftp" ]; then		
		logmsg "INFO perform the restore"
		logprintf "`gettext 'Performing the restore, please wait...'`\n"
		logoutput $RUN_AS_ROOT transferrt.sh $SFTPSERVER $FILENAME $USER $PASSWORD
		if [ $? -ne 0 ]; then
			logmsg "ERROR Failed to copy the backup file from the SFTP server."
			logprintf "`gettext 'Failed to copy the backup file from the SFTP server.'`\n" 
			exit_cleanup 25
		fi
	elif [ "$TYPE" = "local" ]; then	
		logprintf "`gettext 'Restoring from local, please wait...'`\n"
		RESTORE="runcommand2 /opt/ibm/directorappliance/bin/restorevm.sh 'local' $FILENAME"
		logoutput $RUN_AS_ROOT $MGMTNETUTIL $RESTORE
		rc=$?
		if [ $rc -ne 0 ]; then
			if [ $rc -eq 11 ]; then
				logprintf "`gettext 'Cannot find the backup file, please check it and try again.'`\n"
				exit_cleanup 26
			else
				logprintf "`gettext 'Failed to copy the backup file from local repository.'`\n"
				exit_cleanup 27
			fi
		fi
	fi

	# decompress the restore file
	logmsg "INFO Extracting the backup"
	logprintf "`gettext 'Extracting the backup...'`\n"
	DECOMPRESS="runcommand2 /opt/ibm/directorappliance/bin/restorevm.sh 'decompress'"	
	logoutput $RUN_AS_ROOT $MGMTNETUTIL $DECOMPRESS
	if [ $? -ne 0 ]; then
		logmsg "ERROR Failed to extract the backup file contents."
		logprintf "`gettext 'Failed to extract the backup file contents.'`\n" 
		exit_cleanup 35
	fi
	# check the manifest file
	logmsg "INFO check the manifest file"
	CHECKMANIFEST="runcommand2 /opt/ibm/directorappliance/bin/restorevm.sh 'manifest'"
	logoutput $RUN_AS_ROOT $MGMTNETUTIL $CHECKMANIFEST
	rc=$?
	logmsg "INFO the return code of restorevm manifest is $rc"
	if [ $rc -ne 0 ]; then
			if [ $rc -eq 11 ]; then
				logmsg "ERROR The restore media is newer than the installed software.  Update to the latest software level before attempting the restore."
				logprintf "`gettext 'The restore media is newer than the installed software.  Update to the latest software level before attempting the restore.'`\n"
				exit_cleanup 26
			else
				logmsg "ERROR The backup file is not applicable for this machine."
				logprintf "`gettext 'The backup file is not applicable for this machine.'`\n"
				exit_cleanup 27
			fi
	fi
	
	# restore uuid from backup manifest file to DirectorVM.xml
	logmsg "INFO restore uuid from backup manifest file to DirectorVM.xml"
	logoutput $RUN_AS_ROOT manifest restoreUUID
	
	if [ $usbCleanFlag -eq 1 ]; then
		# umount drive in host, sync in host, drop all file system caches in guest
		logmsg "INFO clean appliance after usb restore"
		logoutput $RUN_AS_ROOT usbUtil clean
	fi

	# call watchdog mgr not to reboot host
	logmsg "INFO call watchdog mgr not to reboot"
	WATCHDOG="runcommand2 /opt/ibm/directorappliance/bin/watchdog-mgr restore /opt/ibm/directorappliance/bin/restorevm.sh 'replace'"
	logoutput $RUN_AS_ROOT $MGMTNETUTIL $WATCHDOG

	logmsg "INFO The system is going to reboot"
	logprintf "`gettext 'The system is going to reboot'`\n"
	logoutput $RUN_AS_ROOT haltmsg
	STOP_VM=stopvm
	logoutput $RUN_AS_ROOT $STOP_VM -t now &

else
	logmsg "ERROR Command not available in this configuration. Contact the product supplier for details."
	logprintf "`gettext 'Command not available in this configuration. Contact the product supplier for details.'`\n"
fi
	
exit_cleanup 0
