#!/bin/sh
#
# Licensed Materials - Property of IBM
#
# (C) COPYRIGHT International Business Machines Corp. 2012
# All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#

# debug flag
# set -x 

# continue on error
 set +e

# cleanup to ensure the files we use will be new
if [ -f /tmp/dbibmdircfg.txt ] 
then rm -f /tmp/dbibmdircfg.txt 
fi
if [ -f /tmp/filelist.txt ] 
then rm -f /tmp/filelist.txt 
fi
# db2 command is in the path
# director is active, db2 is also to be active
# root is not authorized to change the db config so
# when running as root (pesh access for testing) we can su
# this configures the db2 instance ibmdir01 so it does not archive
# root is not authorized to the command so use su and db2usr01 
su db2usr01 -c "db2 update db cfg for ibmdir01 using LOGARCHMETH1 OFF" 

# To re-enable archiving, 
# su db2usr01 -c "db2 update db cfg for ibmdir01 using LOGARCHMETH1 DISK:/home/db2inst1/ArchiveDest/"
# if the DB was started without archiving (and the above command then ran) before the DB can be used, the DB has to be saved to be useable.
# su db2usr01 -c "db2 backup db ibmdir01 to /home/db2inst1/ArchiveDest/"

# Now we need to clean up, and to be safe we should not remove any file DB2 consideres active
# get the current configuration for the db2 instance
db2 get db cfg for ibmdir01 >> /tmp/dbibmdircfg.txt

# find the  First active log file line in the file, use "=" as a seperator and take the 2nd field
# if archiving has already been disabled, the string will not be in the db2 config file
# if no string or file is listed in the db config, will have an error message on our compare but NO files will be removed (desired behavior)
firstActiveFile=`cat /tmp/dbibmdircfg.txt | grep "First active log file" | echo $(cut -d"=" -f2)`

# lets get to where the archive log files are located
cd /mirror0/director/db2inst1/ArchiveDest/db2inst1/IBMDIR01/NODE0000/C0000000 

#now get a listing of all the *.LOG files
ls *.LOG >> /tmp/filelist.txt

# we have the first active *.LOG file name. 
# We now want to remove all *.LOG files older then our first one in use.
# comparison of the same file will NOT delete the file

while read filename
do
if test $filename -ot $firstActiveFile
then
# echo is only for unit tst
echo $filename is older the $firstActiveFile, delete $filename
rm -f $filename 
else
echo Do nothing: $filename is newer then $firstActiveFile
fi
done</tmp/filelist.txt 

#all done, lets clean up our files 
rm -f /tmp/dbibmdircfg.txt
rm -f /tmp/filelist.txt

 
