#!/bin/bash

LOC=/opt/ibm/director/vmi/bin/
EXP=$LOC/sshbk.exp
MON=$LOC/monitor

cleanup() {
  # kill any remaining processes
  # check to see if the pid exists before killing
  kill -0 $pid_exp 2>/dev/null && kill $pid_exp 2>/dev/null
  kill -0 $pid_tar 2>/dev/null && kill $pid_tar 2>/dev/null
  kill -0 $pid_prl 2>/dev/null && kill $pid_prl 2>/dev/null

  # wait will cause use to pause until the pid exits
  # since we already killed them, it shouldn't wait
  # the return code is the same as what was originally
  # returned.
  wait $pid_tar
  rc_tar=$?

  wait $pid_exp
  rc_exp=$?

  wait $pid_prl
  rc_prl=$?

  # remove the pipes
  rm -rf $dir

  # Or the return codes, if any of them fail we would not exit 0
  let rc="$rc_tar | $rc_exp | $rc_prl"
  exit $rc
}

if [ $# -ne 4 ]; then
  echo "Incorrect number of variables"
  exit 2
fi

# run cleanup on the following signals
trap cleanup SIGINT SIGTERM SIGKILL

# mktemp can't make pipes
# make a directory and set create the pipes
# in there
dir=$(mktemp -d /tmp/backup.XXXXXXXX)
if [ $? -ne 0 ]; then
  echo "Cannot create backup directory"
  exit 1
fi

if [ ! -p $dir/fifo1 ]; then
  mkfifo $dir/fifo1
fi

if [ ! -p $dir/fifo2 ]; then
  mkfifo $dir/fifo2
fi

# output the tar to fifo1 
ssh root@10.3.0.1 'tar czpSf - /mnt/HostVG/vmsnapshot' > $dir/fifo1 &
pid_tar=$!

# start the remote command to read from fifo2
$EXP $3 $1 $4 $dir/fifo2 $2 &
pid_exp=$!

# monitor will read from fifo1 and output to fifo2
$MON $dir/fifo1 3>&1 1>$dir/fifo2 2>&3- &
pid_prl=$!

# Wait until the perl script finishes
# we background it so we can get the pid for later usage
wait $pid_prl

# Wait 10 seconds, to allow things to shutdown
sleep 10

cleanup
